%% Doing Economics with the Computer
% *Lecture 4: SOLUTIONS* $\\$
% Harris Dellas $\\$
% Lucas Kyriacou

%% Matlab Application: Financial Evaluation 1
% In this lecture we will explain how to use MATLAB to solve 
% amortization problems, interest compounding and defaulting on a mortgage.  

%% Compulsory Readings
% Up to now, you should have read chapters 1 - 6 in Attaway (2019). The book is provided on Ilias. 

%% Further Exercises
% Up to now, you should have solved the practice questions in chapters 1 - 6 in Attaway (2019). 
% Solutions to the practice questions in chapter 3-6 are available on Ilias.

%% Housekeeping
% When you write a script, you should include these commands at the start.

clear all % Deletes memory
close all % closes open windows (figures etc.)
clc       % clears command window 


%% Number format
% Change number format
format bank % Format numbers in currency format with 2 digits after the decimal point

%% Fixed Rate Mortgage Payments
% Let us consider a fixed rate mortgage, that is, a mortgage that carries a 
% fixed interest rate over its lifetime.  We would like to understand how 
% mortgage payments relate to the initial amount and the evolution of 
% the principal, and the role of the interest rate and the length of the 
% loan period. Let us assume that payments are made monthly until the 
% principal has been paid out and that all the payments are of the same size.

%% Definitions
% * $T =$  maturity of loans (number of years) 
% * $N =T*12=$ total number of payments
% * $n =$ is number of payment
% * $p(t) =$ principal, $t=1, 2, ...,N$, 
% * $A =$ initial principal 
% * $R =$ constant, monthly payment
% * $ra =$ annual interest rate
% * $r =ra/12=$ monthly interest rate

%% Example:
% A 20-year loan of size $A= 350000$, at an annual interest of $ra=0.05$, that is $5\%$. 
% The principal then evolves according to $p(n+1)=(1+r)p(n) - R$.
% We can write the principal on the final date as (see table) $\\\\$
% $p(N)=A*(1+r)^N-R*\sum_{j=0}^{N-1} (1+r)^j$. $\\\\$
% The principal on the final date must be zero (the loan will have been fully repaid), 
% that is $p(N)=0$. Using this and solving for R 
% (utilizing the definition of the sum of a finite geometric series) gives
% the required amount of monthly  payment R: $\\\\$
% $R= \frac{r*A*(1+r)^N}{(1+r)^N-1}$.

% \input{Table1}

%% Exercises:
%% A. Fixed Rate Mortgage Payments
%% a) 
% Suppose that $T=30, ra=0.05, A=400000$. Using MATLAB determine the required monthly payments R. 
% How much principal is left after 10 years? After 20 years? 
% Use a for loop to answer the latter questions. 
% Do it both on line (i.e. 'in script form') and using a function file.

%% a.i) Script form
A  = 400000; 
T  = 30; 
ra = 0.05;
N  = 12*T;
r  = ra/12;

% Determine monthly payment: 
R=(r*A*(1+r)^N)/((1+r)^N-1);

% Determine principal: 
p=[]; 
p(1)=A*(1+r)-R;

for n=1:N-1
    p(n+1)=(1+r)*p(n)-R;
end

% Monthly Payment
fprintf('Monthly Payment: %.3f USD \n',R);

% Principal left after 10 years
fprintf('Principal left after 10 years: %.3f USD \n', p(120));

% Principal left after 20 years
fprintf('Principal left after 20 years: %.3f USD \n', p(240));

% You can also access the principals for 10 and 20 years directly: 
 [p(120) p(240)] 

%% a.ii) Function file
% Write the function file, pay.m that calculates the required monthly payments and the principal for each month. 
% Give the parameter values from above and call the function. The result
% should be the same as in a.i). 

A  = 400000; 
T  = 30; 
ra = 0.05;
N  = 12*T;
r  = ra/12;

[p, R] = pay(A,r,N);

% Monthly Payment
fprintf('Monthly Payment: %.3f USD \n',R);

% Principal left after 10 years
fprintf('Principal left after 10 years: %.3f USD \n', p(120));

% Principal left after 20 years
fprintf('Principal left after 20 years: %.3f USD \n', p(240));


%% b) 
% Let the annual interest rate now be higher, namely 0.06 and answer part (a) with the function file pay.m.
clear; 

A  = 400000; 
T  = 30; 
ra = 0.06; % Higher interest rate
N  = 12*T;
r  = ra/12;

[p, R] = pay(A,r,N);

% Monthly Payment
fprintf('Monthly Payment: %.3f USD \n',R);

% Principal left after 10 years
fprintf('Principal left after 10 years: %.3f USD \n', p(120));

% Principal left after 20 years
fprintf('Principal left after 20 years: %.3f USD \n', p(240));


%% c) 
% Suppose that given your income, you can afford to make a maximum monthly payment $R=500$. 
% If the annual interest rate is $ra=0.05$ what is the maximum amount of a 20 year loan that you can afford to borrow? Of a 30 year loan?

%% c.i) Using a script file: 20 year loan
R  = 500;
T  = 20;
ra = 0.05;
N  = 12*T;
r  = ra/12;

A=fzero(@(A) R-(r*A*(1+r)^N)/((1+r)^N-1), 200000); 

% Maximum amount of a 20 year loan that you can afford to borrow
fprintf('Maximum amount of a 20 year loan that you can afford to borrow: %.3f USD \n', A);

% Same as if we were to rewrite the formula for R: 
A_alt = R*((1+r)^N-1)/(r*(1+r)^N);
fprintf('Maximum amount of a 20 year loan that you can afford to borrow: %.3f USD \n', A_alt);

%% c.i) Using a script file: 30 year loan
R  = 500;
T  = 30;
ra = 0.05;
N  = 12*T;
r  = ra/12;

A=fzero(@(A) R-(r*A*(1+r)^N)/((1+r)^N-1), 200000);

% Maximum amount of a 30 year loan that you can afford to borrow
fprintf('Maximum amount of a 30 year loan that you can afford to borrow: %.3f USD \n', A);


%% c.ii) Using a function file: 30 year loan
R     = 500;
T     = 30;
ra    = 0.05;
param = [R,T,ra];
A0    = 200000; %Initial guess for A (the amount of the maximum loan)
% options=[1e9;1e-6;1e-8;1e-3;0];

A=fcsolve(@heidi,A0,[],param);
% See the function file heidi.m    

% Maximum amount of a 30 year loan that you can afford to borrow
fprintf('Maximum amount of a 30 year loan that you can afford to borrow: %.3f USD \n', A);

%% d)
% Suppose that you have an arrangement that during the first 5 years of the 30 year
% mortgage of 400'000 you only have to pay the interest due and nothing of the capital. 
% What is the monthly payment you will have to make over the remaining 25 years? 
% Is it smaller or larger than if you did not have this deal? Compute the principal 
% after 10 years. Plot the evolution of the principal with and without the deal in the same graph.

% With the deal:
A=400000; T=30; ra=0.05; N=12*T; r=ra/12;
R=(r*A*(1+r)^(N-60))/((1+r)^(N-60)-1); % pay the capital over 25 years

p=[]; 
p(61)=A*(1+r)-R;

for n=61:N-1;
    p(n+1)=(1+r)*p(n)-R;
end

p(120)
pb=p;

% Without the deal: 
A=400000; T=30; ra=0.05; N=12*T; r=ra/12;
[pa, R] = pay(A,r,N);

plot(1:N,[pa; pb])

%% e) 
% Suppose now that you pay nothing during the first 5 years and then you pay 
% 500 over the following 5 years. What must the monthly payment R over the remaining 20 years 
% be in order for the mortgage to be fully repaid?

A=400000; T=30; ra=0.05; N=12*T; r=ra/12; p=[]; 

% Pay R1 = 500 from year 5 to 10
R1   = 500; 
A5   = A*(1+r)^60;
p(61)= A5*(1+r)-R1;

for n=61:120
    p(n+1)=(1+r)*p(n)-R1;
end

% Payment over the remaining 20 years
R=(r*p(120)*(1+r)^(N-120))/((1+r)^(N-120)-1); % pay the capital over 20 years

fprintf('Monthly payment over the remaining 20 years: %.3f USD \n', R);

%% e.i) 
% Suppose that the bank unexpectedly forgives the remaining amount of the loan once the 
% principal has fallen below 50000. After how many months will this happen?

A=400000;T=30;ra=0.05; N=12*T; r=ra/12;
R=(r*A*(1+r)^N)/((1+r)^N-1);

p=[];
p(1)=A*(1+r)-R;

for n=1:N-1
    p(n+1)=(1+r)*p(n)-R;
    if p(n+1) < 50000
        break; 
    end
end

l = size(p)
p(l) 
% p(l-1)

fprintf('Principal has fallen below 50000 after: %.f Months \n', l(2));

%% e.ii) 
% Find an alternative way to solve the task above (use a while loop)

A=400000; T=30; ra=0.05; N=12*T; r=ra/12;
R=(r*A*(1+r)^N)/((1+r)^N-1);

p=[]; p0=A*(1+r)-R; crit=1; n=1; 

while crit>0
    p=(1+r)*p0-R;  
    crit=p-50000;
    p0=p;
    n=n+1;
end

fprintf('Principal has fallen below 50000 after: %.f Months \n', n);

%% B. Flexible Rate Mortgage Payment
% Let us now consider a flexible, 30 year mortgage, that is, a mortgage
% for which the interest rate changes from one month to the next. 
% Let the monthly rate be r(n) and suppose, for simplicity, that the path of
% interest rates is known when the mortgage is issued and that the borrower
% still makes a fixed payment of 2000 every month.  At the end of the 30 years
% period, the borrower pays the balance to the creditor. How much is this balance?
% Let $T=30,  A=400000$ so that $N=360$, $R=2000$  and r be in the interval [a,b] of a uniform distribution, 
% $a=0.003,b=0.005$, so that  $r = a + (b-a)*rand(360,1)$.

%% Exercises:
%% B. Flexible Rate Mortgage Payments
%% a)  
% Using MATLAB determine the amount of principal left after: 10 years;  after 30 years.

A=400000;T=30; N=12*T; a=0.003; b=0.005; R=2000; r = a + (b-a)*rand(360,1);
p=[]; 

p(1)=A*(1+r(1))-R;

for n=1:N-1
    p(n+1)=(1+r(n+1))*p(n)-R;
end

% Principal left after 10 years
fprintf('Principal left after 10 years: %.3f USD \n', p(10*12));

% Principal left after 30 years
fprintf('Principal left after 30 years: %.3f USD \n', p(12*30));

%% b.i) 
% Now suppose that in each period you pay the full interest cost plus one 
% tenth of the principal owed in that period. How much will you owe 
% after 5 years? How many months will it take you before your principal drops below 50000?

A=400000;T=30; N=12*T; a=0.003; b=0.005; r = a + (b-a)*rand(360,1);
p=[];p(1)=A*(1+r(1))-0.1*A; 

for n=1:N-1
    p(n+1)=(1+r(n+1))*p(n)-0.1*p(n);
end

% How much will you owe after 5 years?
fprintf('How much will you owe after 5 years: %.3f USD \n', p(5*12));

% How many months will it take you before your principal drops below 50000?
k=p(p>50000);
[i, j] = size(k);
fprintf('Months before principal drops below 50000: %.f Months \n', j);

% Note: After 20 Months, principal is: 
fprintf('\nNote: \n')
fprintf('Principal after 20 Months: %.f USD \n',  p(20));

% Note: After 21 Months, principal is: 
fprintf('Principal after 21 Months: %.f USD \n',  p(21));
%% b.ii) 
% Find an alternative way to solve the task above
[~, j]=(min(abs(p-50000)));
fprintf('Months before principal drops below 50000: %.f Months \n', j-1);